const { exec } = require('child_process');
const TeslaChargingLogger = require('./tesla.charging-logger');
const path = require('path');
const fs = require('fs');

class TeslaControlBLE {
    static config = {
        vin: null,
        keyFile: null,
        executablePath: "tesla-control", // Chemin par défaut (utilise PATH)
        commandTimeout: 15000,
        wakeupWaitTime: 15000,
        commandDelay: 4000
    };

    // Propriété pour suivre la dernière exécution de commande
    static lastCommandTime = 0;
    static commandQueue = [];  // File d'attente des commandes
    static isProcessingQueue = false;  // Indicateur que la file est en cours de traitement

    // États des éléments du véhicule
    static CLOSURE_STATE = {
        CLOSED: "CLOSURESTATE_CLOSED",
        OPEN: "CLOSURESTATE_OPEN"
    };

    static VEHICLE_LOCK_STATE = {
        LOCKED: "VEHICLELOCKSTATE_LOCKED",
        UNLOCKED: "VEHICLELOCKSTATE_UNLOCKED",
        INTERNAL_LOCKED: "VEHICLELOCKSTATE_INTERNAL_LOCKED",
        SELECTIVE_UNLOCKED: "VEHICLELOCKSTATE_SELECTIVE_UNLOCKED"
    };

    static VEHICLE_SLEEP_STATUS = {
        UNKNOWN: "VEHICLE_SLEEP_STATUS_UNKNOWN",
        AWAKE: "VEHICLE_SLEEP_STATUS_AWAKE",
        ASLEEP: "VEHICLE_SLEEP_STATUS_ASLEEP"
    };

    static USER_PRESENCE = {
        UNKNOWN: "VEHICLE_USER_PRESENCE_UNKNOWN",
        NOT_PRESENT: "VEHICLE_USER_PRESENCE_NOT_PRESENT",
        PRESENT: "VEHICLE_USER_PRESENCE_PRESENT"
    };

    // Categories for the state command
    static STATE_CATEGORIES = {
        LOCATION: 'location',
        CLOSURES: 'closures',
        MEDIA: 'media',
        MEDIA_DETAIL: 'media-detail',
        SOFTWARE_UPDATE: 'software-update',
        CHARGE: 'charge',
        CLIMATE: 'climate',
        PRECONDITION_SCHEDULE: 'precondition-schedule',
        TIRE_PRESSURE: 'tire-pressure',
        PARENTAL_CONTROLS: 'parental-controls',
        DRIVE: 'drive',
        CHARGE_SCHEDULE: 'charge-schedule'
    };

    /**
     * Initialize the Tesla BLE Control service
     * @param {Object} config - Configuration object
     * @returns {boolean} Success status
     */
    static init(config) {
        try {
            TeslaChargingLogger.writeToLog("🚗 TESLA BLE CONTROL: Initializing service");
            
            if (!config?.tesla) {
                TeslaChargingLogger.writeToLog("🚗❌ TESLA BLE CONTROL: Missing Tesla configuration");
                return false;
            }
            
            this.config.vin = config.tesla.vin;

            if (config.tesla.BLEexecutablePath) {
                this.config.executablePath = config.tesla.BLEexecutablePath;
                TeslaChargingLogger.writeToLog(`🚗 TESLA BLE CONTROL: Using custom executable path: ${this.config.executablePath}`);
            }
            
            // Configuration du délai entre commandes
            if (config.tesla.BLECommandDelay !== undefined) {
                this.config.commandDelay = config.tesla.BLECommandDelay;
                TeslaChargingLogger.writeToLog(`🚗 TESLA BLE CONTROL: Using custom command delay: ${this.config.commandDelay}ms`);
            }
            
            // Configuration du timeout de commande
            if (config.tesla.BLECommandTimeout !== undefined) {
                this.config.commandTimeout = config.tesla.BLECommandTimeout;
                TeslaChargingLogger.writeToLog(`🚗 TESLA BLE CONTROL: Using custom command timeout: ${this.config.commandTimeout}ms`);
            }
            
            // Configuration du temps d'attente après réveil
            if (config.tesla.BLEWakeupWaitTime !== undefined) {
                this.config.wakeupWaitTime = config.tesla.BLEWakeupWaitTime;
                TeslaChargingLogger.writeToLog(`🚗 TESLA BLE CONTROL: Using custom wakeup wait time: ${this.config.wakeupWaitTime}ms`);
            }
            
            // Utiliser le chemin de clé défini dans la configuration
            if (config.tesla.BLEkeyFile) {
                this.config.keyFile = path.resolve(process.cwd(), config.tesla.BLEkeyFile);
                TeslaChargingLogger.writeToLog(`🚗 TESLA BLE CONTROL: Using key file from config: ${this.config.keyFile}`);
            } else {
                // Fallback sur un chemin par défaut si non défini dans la config
                const confDir = path.join(process.cwd(), 'assets', 'conf');
                this.config.keyFile = path.join(confDir, 'BLE-private.pem');
                TeslaChargingLogger.writeToLog(`🚗 TESLA BLE CONTROL: Using default key file path: ${this.config.keyFile}`);
            }
            
            // Check if key file exists
            if (!fs.existsSync(this.config.keyFile)) {
                TeslaChargingLogger.writeToLog(`🚗❌ TESLA BLE CONTROL: Key file not found at ${this.config.keyFile}`);
                return false;
            }
            
            if (!this.config.vin) {
                TeslaChargingLogger.writeToLog("🚗❌ TESLA BLE CONTROL: VIN is required");
                return false;
            }
            
            TeslaChargingLogger.writeToLog(`🚗 TESLA BLE CONTROL: Initialized with VIN ${this.config.vin}`);
            return true;
        } catch (error) {
            TeslaChargingLogger.writeToLog(`🚗❌ TESLA BLE CONTROL: Initialization failed: ${error.message}`);
            return false;
        }
    }

/**
 * Execute a Tesla control command via BLE
 * @param {string} command - Command to execute
 * @param {Array} args - Command arguments
 * @returns {Promise<string|Object>} Command result
 */
static async execute(command, args = []) {
    // Au lieu d'exécuter directement, ajouter à la file d'attente
    return new Promise((resolve, reject) => {
        this.commandQueue.push({ command, args, resolve, reject });
        TeslaChargingLogger.writeToLog(`🚗🔁 TESLA BLE: Commande '${command}' mise en file d'attente`);
        
        // Démarrer le traitement si ce n'est pas déjà fait
        if (!this.isProcessingQueue) {
            this.processCommandQueue();
        }
    });
}

/**
 * Process commands from the queue one by one
 * @private
 */
static async processCommandQueue() {
    if (this.isProcessingQueue) return;
    this.isProcessingQueue = true;
    
    try {
        while (this.commandQueue.length > 0) {
            const task = this.commandQueue.shift();
            const { command, args, resolve, reject } = task;
            
            try {
                // Vérifier s'il faut attendre avant d'exécuter une nouvelle commande
                const now = Date.now();
                const timeSinceLastCommand = now - this.lastCommandTime;
                
                if (this.lastCommandTime > 0 && timeSinceLastCommand < this.config.commandDelay) {
                    // Calculer le temps d'attente nécessaire
                    const waitTime = this.config.commandDelay - timeSinceLastCommand;
                    TeslaChargingLogger.writeToLog(`🚗⏱️ TESLA BLE: Attente de ${waitTime}ms entre les commandes pour éviter les conflits de connexion`);
                    
                    // Attendre le délai nécessaire
                    await this.sleep(waitTime);
                }
                
                // Exécuter la commande
                const result = await this._executeCommand(command, args);
                resolve(result);
            } catch (error) {
                reject(error);
            }
        }
    } finally {
        this.isProcessingQueue = false;
    }
}

/**
 * Internal implementation of command execution
 * @private
 */
static _executeCommand(command, args = []) {
    return new Promise((resolve, reject) => {
        try {
            TeslaChargingLogger.writeToLog(`🚗 TESLA BLE: Executing '${command}' with args: ${args.join(' ')}`);
            
            const teslaControlPath = this.config.executablePath;
            
            // Build command with required parameters
            const cmdArgs = [
                `-vin ${this.config.vin}`,
                `-ble`,
                `-key-file ${this.config.keyFile}`,
                command,
                ...args
            ].join(' ');
            
            const fullCommand = `${teslaControlPath} ${cmdArgs}`;
            TeslaChargingLogger.writeToLog(`🚗 TESLA BLE: Running: ${fullCommand}`);
            
            // Execute command
            exec(fullCommand, { timeout: this.config.commandTimeout }, (error, stdout, stderr) => {
                // Mettre à jour le timestamp APRÈS l'exécution complète de la commande
                this.lastCommandTime = Date.now();
                
                // Gérer les erreurs connues avec des messages spécifiques
                if (error) {
                    // [tout le code de gestion d'erreur existant]
                    if (error.message.includes('already connected to the maximum number of BLE devices')) {
                        const customError = new Error('La voiture est déjà connectée au nombre maximum d\'appareils BLE');
                        customError.code = 'MAX_BLE_CONNECTIONS';
                        TeslaChargingLogger.writeToLog(`🚗❌ TESLA BLE: ${customError.message}`);
                        reject(customError);
                        return;
                    }
                    
                    // Copier tous les autres blocs if existants de gestion d'erreur
                    
                    TeslaChargingLogger.writeToLog(`🚗❌ TESLA BLE: Error: ${error.message}`);
                    reject(error);
                    return;
                }
                
                // Vérifier les erreurs dans stderr
                if (stderr && stderr.trim().length > 0) {
                    // [code existant]
                }
                
                // Une sortie vide est généralement un succès pour les commandes sans retour
                if (!stdout || stdout.trim() === '') {
                    TeslaChargingLogger.writeToLog(`🚗✅ TESLA BLE: Commande exécutée avec succès (pas de retour)`);
                    resolve({ success: true, command: command });
                    return;
                }
                
                TeslaChargingLogger.writeToLog(`🚗✅ TESLA BLE: Success: ${stdout.substring(0, 100)}${stdout.length > 100 ? '...' : ''}`);
                
                try {
                    // Return as JSON if possible
                    const result = stdout.includes('{') ? JSON.parse(stdout) : stdout;
                    resolve(result);
                } catch {
                    resolve(stdout);
                }
            });
        } catch (error) {
            // Mettre à jour le timestamp en cas d'erreur aussi
            this.lastCommandTime = Date.now();
            TeslaChargingLogger.writeToLog(`🚗❌ TESLA BLE: Execution error: ${error.message}`);
            reject(error);
        }
    });
}

    /**
     * Sleep function - attendre un certain temps
     * @param {number} ms - Nombre de millisecondes à attendre
     * @returns {Promise<void>}
     */
    static sleep(ms) {
        return new Promise(resolve => setTimeout(resolve, ms));
    }

    /**
     * Get basic vehicle state without waking it up
     * @returns {Promise<Object>} Body controller state or null if failed
     */
    static async getBodyControllerState() {
        try {
            TeslaChargingLogger.writeToLog("🚗 TESLA BLE: Getting body controller state");
            return await this.execute('body-controller-state');
        } catch (error) {
            TeslaChargingLogger.writeToLog(`🚗⚠️ TESLA BLE: Failed to get body controller state: ${error.message}`);
            return null;
        }
    }

    /**
     * Check if the vehicle is available (without waking it)
     * @returns {Promise<boolean>} true if vehicle is reachable
     */
    static async isVehicleReachable() {
        const state = await this.getBodyControllerState();
        if (!state) return false;
        
        TeslaChargingLogger.writeToLog(`🚗 TESLA BLE: Vehicle state: lock=${state.vehicleLockState}, sleep=${state.vehicleSleepStatus}, user=${state.userPresence}`);
        return true;
    }

    /**
     * Check if the vehicle is awake
     * @returns {Promise<boolean>} true if vehicle is awake
     */
    static async isVehicleAwake() {
        const state = await this.getBodyControllerState();
        if (!state) return false;
        
        const isAwake = state.vehicleSleepStatus === this.VEHICLE_SLEEP_STATUS.AWAKE;
        TeslaChargingLogger.writeToLog(`🚗 TESLA BLE: Vehicle is ${isAwake ? 'awake' : 'asleep'}`);
        return isAwake;
    }

    /**
     * Check if the charge port is open
     * @returns {Promise<boolean>} true if charge port is open
     */
    static async isChargePortOpen() {
        const state = await this.getBodyControllerState();
        if (!state || !state.closureStatuses) return false;
        
        const isOpen = state.closureStatuses.chargePort === this.CLOSURE_STATE.OPEN;
        TeslaChargingLogger.writeToLog(`🚗 TESLA BLE: Charge port is ${isOpen ? 'open' : 'closed'}`);
        return isOpen;
    }

    /**
     * Check if the vehicle is locked
     * @returns {Promise<boolean>} true if vehicle is locked
     */
    static async isVehicleLocked() {
        const state = await this.getBodyControllerState();
        if (!state) return false;
        
        const isLocked = state.vehicleLockState === this.VEHICLE_LOCK_STATE.LOCKED || 
                        state.vehicleLockState === this.VEHICLE_LOCK_STATE.INTERNAL_LOCKED;
        
        TeslaChargingLogger.writeToLog(`🚗 TESLA BLE: Vehicle is ${isLocked ? 'locked' : 'unlocked'}`);
        return isLocked;
    }

    /**
     * Check if any doors or trunks are open
     * @returns {Promise<Object>} Object with information about open closures
     */
    static async getOpenClosures() {
        const state = await this.getBodyControllerState();
        if (!state || !state.closureStatuses) return null;
        
        const openClosures = {};
        let hasOpenClosures = false;
        
        for (const [key, value] of Object.entries(state.closureStatuses)) {
            if (value === this.CLOSURE_STATE.OPEN) {
                openClosures[key] = true;
                hasOpenClosures = true;
            }
        }
        
        return { hasOpenClosures, openClosures };
    }

    /**
     * Vérifie la disponibilité de la voiture sans la réveiller
     * Cette méthode est la première étape dans toute séquence de commandes
     * @returns {Promise<Object|null>} État du contrôleur ou null si non disponible
     */
    static async checkVehicleAvailability() {
        TeslaChargingLogger.writeToLog("🚗 TESLA BLE: Vérification de la disponibilité du véhicule");
        
        try {
            // Utiliser body-controller-state qui fonctionne même si la voiture est endormie
            const state = await this.getBodyControllerState();
            if (!state) {
                TeslaChargingLogger.writeToLog("🚗❌ TESLA BLE: Véhicule non disponible");
                return null;
            }
            
            const isAwake = state.vehicleSleepStatus === this.VEHICLE_SLEEP_STATUS.AWAKE;
            TeslaChargingLogger.writeToLog(`🚗 TESLA BLE: Véhicule disponible, état: ${isAwake ? 'réveillé' : 'endormi'}`);
            
            return state;
        } catch (error) {
            TeslaChargingLogger.writeToLog(`🚗❌ TESLA BLE: Erreur lors de la vérification de disponibilité: ${error.message}`);
            return null;
        }
    }

    /**
     * Mise à jour intelligente des données véhicule
     * Combine les données MQTT, API et BLE selon leur disponibilité
     * @param {Object} mqttData - Données provenant de MQTT (si disponibles)
     * @returns {Promise<Object>} Données complètes du véhicule
     */
    static async updateVehicleData(mqttData = null) {
        TeslaChargingLogger.writeToLog("🔄 TESLA BLE: Mise à jour intelligente des données véhicule");
        
        // Étape 1: Vérifier si la voiture est disponible sans la réveiller
        const basicState = await this.checkVehicleAvailability();
        if (!basicState) {
            TeslaChargingLogger.writeToLog("🚗❌ TESLA BLE: Véhicule hors de portée BLE, utilisation exclusive des données MQTT");
            return mqttData || { source: 'none', timestamp: Date.now() };
        }
        
        // Créer un objet de données enrichi avec les informations de base
        const enrichedData = {
            basic_state: basicState,
            timestamp: Date.now(),
            source: 'ble_basic'
        };
        
        // Si la voiture est endormie et qu'on a des données MQTT, les utiliser sans réveiller la voiture
        if (basicState.vehicleSleepStatus !== this.VEHICLE_SLEEP_STATUS.AWAKE && mqttData) {
            TeslaChargingLogger.writeToLog("🚗 TESLA BLE: Véhicule endormi, combinaison avec données MQTT sans réveil");
            return { ...mqttData, basic_state: basicState, source: 'mqtt_and_ble_basic' };
        }
        
        // Si la voiture est réveillée ou si on n'a pas de données MQTT, réveiller la voiture et obtenir des données détaillées
        if (await this.ensureConnected()) {
            try {
                // Obtenir l'état de charge détaillé
                const chargeData = await this.getDetailedState(this.STATE_CATEGORIES.CHARGE);
                if (chargeData) {
                    enrichedData.charge_state = chargeData.chargeState;
                    enrichedData.source = 'ble_detailed';
                }
                
                // On pourrait ajouter d'autres catégories au besoin (climate, location, etc.)
                
                TeslaChargingLogger.writeToLog("✅ TESLA BLE: Données détaillées récupérées avec succès");
                return enrichedData;
            } catch (error) {
                TeslaChargingLogger.writeToLog(`⚠️ TESLA BLE: Échec récupération données détaillées: ${error.message}`);
                return { ...(mqttData || {}), basic_state: basicState, source: 'hybrid_fallback' };
            }
        }
        
        // Si on ne peut pas réveiller la voiture, retourner les données de base
        return enrichedData;
    }

    /**
     * Get detailed state information for a specific category
     * This works even when the vehicle is asleep for some categories
     * @param {string} category - One of the categories from STATE_CATEGORIES
     * @returns {Promise<Object>} Category state data
     */
    static async getDetailedState(category) {
        TeslaChargingLogger.writeToLog(`🚗 TESLA BLE: Getting detailed state for category: ${category}`);
        try {
            const result = await this.execute('state', [category]);
            return result;
        } catch (error) {
            TeslaChargingLogger.writeToLog(`🚗⚠️ TESLA BLE: Failed to get ${category} state: ${error.message}`);
            throw error;
        }
    }

    /**
     * Get detailed charging state information
     * @returns {Promise<Object>} Charge state data
     */
    static async getChargeState() {
        try {
            const result = await this.getDetailedState(this.STATE_CATEGORIES.CHARGE);
            return result.chargeState;
        } catch (error) {
            TeslaChargingLogger.writeToLog(`🚗⚠️ TESLA BLE: Failed to get charge state: ${error.message}`);
            return null;
        }
    }

    /**
     * Get current battery level
     * @returns {Promise<number|null>} Battery level percentage or null if failed
     */
    static async getBatteryLevel() {
        try {
            const chargeState = await this.getChargeState();
            if (chargeState && chargeState.batteryLevel !== undefined) {
                TeslaChargingLogger.writeToLog(`🚗 TESLA BLE: Battery level is ${chargeState.batteryLevel}%`);
                return chargeState.batteryLevel;
            }
            return null;
        } catch (error) {
            TeslaChargingLogger.writeToLog(`🚗⚠️ TESLA BLE: Failed to get battery level: ${error.message}`);
            return null;
        }
    }

    /**
     * Check if vehicle is currently charging
     * @returns {Promise<boolean>} true if charging
     */
    static async isCharging() {
        try {
            const chargeState = await this.getChargeState();
            if (!chargeState || !chargeState.chargingState) return false;
            
            const isCharging = 'Charging' in chargeState.chargingState;
            TeslaChargingLogger.writeToLog(`🚗 TESLA BLE: Vehicle is ${isCharging ? 'charging' : 'not charging'}`);
            return isCharging;
        } catch (error) {
            TeslaChargingLogger.writeToLog(`🚗⚠️ TESLA BLE: Failed to check charging status: ${error.message}`);
            return false;
        }
    }

    /**
     * Get estimated time remaining to complete charging
     * @returns {Promise<number|null>} Minutes remaining or null if failed/not charging
     */
    static async getChargeTimeRemaining() {
        try {
            const chargeState = await this.getChargeState();
            if (!chargeState) return null;
            
            if ('minutesToFullCharge' in chargeState) {
                TeslaChargingLogger.writeToLog(`🚗 TESLA BLE: Minutes to full charge: ${chargeState.minutesToFullCharge}`);
                return chargeState.minutesToFullCharge;
            }
            return null;
        } catch (error) {
            TeslaChargingLogger.writeToLog(`🚗⚠️ TESLA BLE: Failed to get charge time remaining: ${error.message}`);
            return null;
        }
    }

    /**
     * Get current charging power
     * @returns {Promise<Object|null>} Charging power info or null if failed
     */
    static async getChargingPower() {
        try {
            const chargeState = await this.getChargeState();
            if (!chargeState) return null;
            
            const power = {
                power: chargeState.chargerPower || 0,
                voltage: chargeState.chargerVoltage || 0,
                current: chargeState.chargerActualCurrent || 0,
                phases: chargeState.chargerPhases || 1
            };
            
            TeslaChargingLogger.writeToLog(`🚗 TESLA BLE: Charging at ${power.power}kW (${power.voltage}V, ${power.current}A)`);
            return power;
        } catch (error) {
            TeslaChargingLogger.writeToLog(`🚗⚠️ TESLA BLE: Failed to get charging power: ${error.message}`);
            return null;
        }
    }

/**
 * Ensures the vehicle is connected and awake before sending commands
 * Uses body-controller-state which works even when the car is asleep
 * @returns {Promise<boolean>} true if vehicle is connected and awake
 */
static async ensureConnected() {
    TeslaChargingLogger.writeToLog("🚗 TESLA BLE: Ensuring vehicle connectivity");
    
    // Obtenir l'état de la voiture avec un seul appel BLE au lieu de deux
    const vehicleState = await this.getBodyControllerState();
    
    // Vérifier si la voiture est joignable
    if (!vehicleState) {
        TeslaChargingLogger.writeToLog("🚗❌ TESLA BLE: Vehicle is not reachable");
        return false;
    }
    
    TeslaChargingLogger.writeToLog(`🚗 TESLA BLE: Vehicle state: lock=${vehicleState.vehicleLockState}, sleep=${vehicleState.vehicleSleepStatus}, user=${vehicleState.userPresence}`);
    
    // Vérifier directement si la voiture est réveillée à partir du même état
    const isAwake = vehicleState.vehicleSleepStatus === this.VEHICLE_SLEEP_STATUS.AWAKE;
    if (isAwake) {
        TeslaChargingLogger.writeToLog("🚗 TESLA BLE: Vehicle is already awake");
        return true;
    }
    
    // Si la voiture est endormie, on tente un wake
    TeslaChargingLogger.writeToLog("🚗 TESLA BLE: Vehicle is asleep, attempting to wake vehicle");
    try {
        await this.execute('wake');
        TeslaChargingLogger.writeToLog(`🚗 TESLA BLE: Wake command sent, waiting ${this.config.wakeupWaitTime}ms`);
        
        // Attente pour laisser à la voiture le temps de se réveiller
        await this.sleep(this.config.wakeupWaitTime);
        
        // Vérifier que la voiture est réveillée avec une nouvelle requête
        // (nécessaire car l'état a changé)
        const newState = await this.getBodyControllerState();
        if (newState && newState.vehicleSleepStatus === this.VEHICLE_SLEEP_STATUS.AWAKE) {
            TeslaChargingLogger.writeToLog("🚗✅ TESLA BLE: Vehicle is now awake");
            return true;
        } else {
            TeslaChargingLogger.writeToLog("🚗❌ TESLA BLE: Vehicle still not awake after wake command");
            return false;
        }
    } catch (error) {
        TeslaChargingLogger.writeToLog(`🚗❌ TESLA BLE: Failed to wake vehicle: ${error.message}`);
        return false;
    }
}

    /**
     * Execute a command with connectivity check
     * @param {string} command - Command to execute
     * @param {Array} args - Command arguments
     * @returns {Promise<Object>} Command result
     */
    static async executeWithConnectivityCheck(command, args = []) {
        if (await this.ensureConnected()) {
            return this.execute(command, args);
        }
        throw new Error("Le véhicule n'est pas accessible");
    }

    // Commandes simples avec vérification de connectivité préalable
    static async wake() {
        // Le wake est spécial, on l'exécute directement
        return this.execute('wake');
    }
    
    static async getState() {
        return this.executeWithConnectivityCheck('state');
    }
    
    // Renommée pour éviter les conflits avec la méthode getBodyControllerState() définie plus haut
    static async getVehicleControllerState() {
        // Cette commande fonctionne même quand la voiture est endormie
        return this.execute('body-controller-state');
    }
    
    static async startCharging() {
        return this.executeWithConnectivityCheck('charging-start');
    }
    
    static async stopCharging() {
        return this.executeWithConnectivityCheck('charging-stop');
    }
    
    static async setChargingAmps(amps) {
        return this.executeWithConnectivityCheck('charging-set-amps', [amps]);
    }
    
    static async setChargeLimit(percent) {
        return this.executeWithConnectivityCheck('charging-set-limit', [percent]);
    }

    static async openChargePort() {
        return this.executeWithConnectivityCheck('charge-port-open');
    }

    static async closeChargePort() {
        return this.executeWithConnectivityCheck('charge-port-close');
    }

    static async lock() {
        return this.executeWithConnectivityCheck('lock');
    }

    static async unlock() {
        return this.executeWithConnectivityCheck('unlock');
    }

    static async climateOn() {
        return this.executeWithConnectivityCheck('climate-on');
    }

    static async climateOff() {
        return this.executeWithConnectivityCheck('climate-off');
    }

    static async setTemperature(temp) {
        return this.executeWithConnectivityCheck('climate-set-temp', [temp]);
    }

    static async flashLights() {
        return this.executeWithConnectivityCheck('flash-lights');
    }

    static async honk() {
        return this.executeWithConnectivityCheck('honk');
    }

    static async ventWindows() {
        return this.executeWithConnectivityCheck('windows-vent');
    }

    static async closeWindows() {
        return this.executeWithConnectivityCheck('windows-close');
    }

    /**
     * Get vehicle location
     * @returns {Promise<Object|null>} Location data or null if failed
     */
    static async getLocation() {
        try {
            const location = await this.getDetailedState(this.STATE_CATEGORIES.LOCATION);
            if (location && location.driveState) {
                const data = {
                    latitude: location.driveState.latitude,
                    longitude: location.driveState.longitude,
                    heading: location.driveState.heading,
                    speed: location.driveState.speed,
                    timestamp: location.driveState.timestamp
                };
                
                TeslaChargingLogger.writeToLog(`🚗 TESLA BLE: Vehicle location: ${data.latitude}, ${data.longitude}`);
                return data;
            }
            return null;
        } catch (error) {
            TeslaChargingLogger.writeToLog(`🚗⚠️ TESLA BLE: Failed to get location: ${error.message}`);
            return null;
        }
    }

    /**
     * Get climate state
     * @returns {Promise<Object|null>} Climate state or null if failed
     */
    static async getClimateState() {
        try {
            const climate = await this.getDetailedState(this.STATE_CATEGORIES.CLIMATE);
            if (climate && climate.climateState) {
                TeslaChargingLogger.writeToLog(`🚗 TESLA BLE: Got climate state`);
                return climate.climateState;
            }
            return null;
        } catch (error) {
            TeslaChargingLogger.writeToLog(`🚗⚠️ TESLA BLE: Failed to get climate state: ${error.message}`);
            return null;
        }
    }

    /**
     * Get vehicle closures status (doors, trunks, etc.)
     * @returns {Promise<Object|null>} Closures state or null if failed
     */
    static async getClosuresState() {
        try {
            const closures = await this.getDetailedState(this.STATE_CATEGORIES.CLOSURES);
            if (closures && closures.vehicleState) {
                TeslaChargingLogger.writeToLog(`🚗 TESLA BLE: Got closures state`);
                return closures.vehicleState;
            }
            return null;
        } catch (error) {
            TeslaChargingLogger.writeToLog(`🚗⚠️ TESLA BLE: Failed to get closures state: ${error.message}`);
            return null;
        }
    }

    /**
     * Check if the current charging rate is above a threshold
     * @param {number} minPower - Minimum power in kW 
     * @returns {Promise<boolean>} true if charging at or above the threshold
     */
    static async isChargingAboveThreshold(minPower) {
        try {
            const power = await this.getChargingPower();
            if (!power) return false;
            
            const isAboveThreshold = power.power >= minPower;
            TeslaChargingLogger.writeToLog(`🚗 TESLA BLE: Charging power ${power.power}kW is ${isAboveThreshold ? 'above' : 'below'} threshold of ${minPower}kW`);
            return isAboveThreshold;
        } catch (error) {
            return false;
        }
    }
}

module.exports = { TeslaControlBLE };