const mqtt = require('mqtt');
const TeslaChargingLogger = require('./tesla.charging-logger');

class TeslaMqttClient {
    // Structure de donnée et état
    static client = null;
    static isConnected = false;
    static vehicleData = {};
    static lastUpdateTime = 0;
    static lastStateUpdate = {};
    static updateCallbacks = [];

    // Initialisation avec configuration
    static init(config) {
        const mqttUrl = `mqtt://${config.host}:${config.port || 1883}`;
        TeslaChargingLogger.writeToLog(`🚀 MQTT: Connexion à ${mqttUrl}`);
        
        // Créer client MQTT avec un ID unique
        const clientId = `energyboard_mqtt_${Date.now()}`;
        this.client = mqtt.connect(mqttUrl, {
            clientId: clientId,
            clean: true
        });
        
        TeslaChargingLogger.writeToLog(`🔑 MQTT: Utilisation de l'ID client: ${clientId}`);
        
        // Définir gestionnaires d'événements
        this.client.on('connect', () => this.onConnect(config));
        this.client.on('message', (topic, message) => this.onMessage(topic, message));
        this.client.on('error', (error) => this.onError(error));
        this.client.on('close', () => this.onClose());
    }
    
    // Gestion connexion réussie
    static onConnect(config) {
        TeslaChargingLogger.writeToLog('✅ MQTT: Connecté au broker TeslaMate');
        this.isConnected = true;
        
        // Souscrire aux topics
        const carId = config.carId || 1;
        this.client.subscribe(`teslamate/cars/${carId}/+`);
        
        // Log pour suivi
        TeslaChargingLogger.writeToLog(`📡 MQTT: Souscription aux données du véhicule ${carId}`);
    }
    
    // Modifier la méthode onMessage pour ajouter l'horodatage à chaque mise à jour
    static onMessage(topic, message) {
        const value = message.toString();
        const topicParts = topic.split('/');
        
        // Vérifier format du topic
        if (topicParts.length < 4) return;
        
        const carId = topicParts[2];
        const attribute = topicParts[3];
        
        // Initialiser structure si nécessaire
        if (!this.vehicleData[carId]) {
            this.vehicleData[carId] = {
                charge_state: {},
                drive_state: {},
                climate_state: {},
                vehicle_state: {},
                timestamp: Date.now(),
                data_source: 'mqtt'
            };
        }
        
        // Toujours mettre à jour le timestamp principal à chaque message reçu
        this.vehicleData[carId]._timestamp = Date.now();
        
        // Convertir valeur selon type
        let processedValue = value;
        if (value === "true" || value === "false") {
            processedValue = value === "true";
        } else if (!isNaN(Number(value)) && value !== "nil") {
            processedValue = Number(value);
        }
        
        // NOUVEAU: Log pour chaque message MQTT reçu, comme dans votre app de test
        const category = this.getCategoryForAttribute(attribute);
        TeslaChargingLogger.writeToLog(`📡 MQTT: [${category}] ${attribute}: ${processedValue}`);
        
        // Stocker par catégorie
        this.updateVehicleData(carId, attribute, processedValue);
        
        // Mettre à jour timestamp
        this.lastUpdateTime = Date.now();
        this.lastStateUpdate[attribute] = Date.now();
        
        // AJOUTÉ: Code de mise en cache des bonnes données
        // Si nous recevons des données importantes et valides, les mettre en cache
        if (this.vehicleData[carId] && 
            this.vehicleData[carId].charge_state && 
            this.vehicleData[carId].charge_state.battery_level > 0) {
            
            // Créer une copie profonde pour éviter les références
            this.lastValidData = this.lastValidData || {};
            this.lastValidData[carId] = JSON.parse(JSON.stringify(this.vehicleData[carId]));
            this.lastValidData[carId].data_source = 'mqtt';
            this.lastValidData[carId].timestamp = Date.now();
        }
        
        // Notifier changements importants
        if (['battery_level', 'state', 'charging_state', 'usable_battery_level', 
             'plugged_in', 'latitude', 'longitude'].includes(attribute)) {
            this.notifyUpdateListeners(carId, attribute, processedValue);
        }
    }

    // NOUVEAU: Méthode pour déterminer la catégorie d'un attribut
    static getCategoryForAttribute(attribute) {
        // Classification par catégorie (comme l'API Tesla)
        if (['battery_level', 'usable_battery_level', 'battery_range', 
            'est_battery_range_km', 'ideal_battery_range_km', 'plugged_in',
            'charge_limit_soc', 'charging_state', 'scheduled_charging_start_time',
            'charger_power', 'charge_port_door_open', 'charger_voltage', 
            'charger_actual_current', 'charge_energy_added', 'time_to_full_charge'].includes(attribute)) {
           return 'charge_state';
        } else if (['latitude', 'longitude', 'speed', 'power', 'shift_state', 'odometer'].includes(attribute)) {
            return 'drive_state';
        } else if (['state', 'locked', 'doors_open', 'windows_open', 'sentry_mode', 'tpms_pressure_fl', 'tpms_pressure_rl'].includes(attribute)) {
            return 'vehicle_state';
        } else if (['inside_temp', 'outside_temp', 'is_climate_on', 'is_preconditioning'].includes(attribute)) {
            return 'climate_state';
        } else {
            return 'divers';
        }
    }
        
    // Organisation des données par catégorie
    static updateVehicleData(carId, attribute, value) {
        const car = this.vehicleData[carId];
        
        // Classification par catégorie (comme l'API Tesla)
        switch(attribute) {
            // Charge state
            case 'battery_level':
            case 'usable_battery_level':
            case 'battery_range':
            case 'est_battery_range_km':
            case 'ideal_battery_range_km':
            case 'plugged_in':
            case 'charge_limit_soc':
            case 'charging_state':
            case 'scheduled_charging_start_time':
            case 'charger_power':
            case 'time_to_full_charge':
            case 'charger_actual_current':
            case 'charger_voltage':
            case 'charge_energy_added':
            case 'charge_current_request':
            case 'charge_port_door_open':
                car.charge_state[attribute] = value;
                break;
                        
                
            // Drive state
            case 'latitude':
            case 'longitude':
            case 'speed':
            case 'power':
            case 'shift_state':
            case 'odometer':
                car.drive_state[attribute] = value;
                
                // Mise à jour de l'objet location si lat/long changent
                if (attribute === 'latitude' || attribute === 'longitude') {
                    car.drive_state.location = {
                        latitude: car.drive_state.latitude || 0,
                        longitude: car.drive_state.longitude || 0
                    };
                }
                break;
                
            // Vehicle state
            case 'state':
                car.vehicle_state[attribute] = value;
                car.state = value; // Pour compatibilité
                break;
                
            // Climate state
            case 'inside_temp':
            case 'outside_temp':
            case 'is_climate_on':
                car.climate_state[attribute] = value;
                break;
                
            // Autres
            default:
                // Données diverses stockées à la racine
                car[attribute] = value;
        }
    }
    
    // Gestion erreur
    static onError(error) {
        TeslaChargingLogger.writeToLog(`❌ MQTT: Erreur - ${error.message}`);
        this.isConnected = false;
    }
    
    // Gestion déconnexion
    static onClose() {
        TeslaChargingLogger.writeToLog('🔌 MQTT: Déconnecté du broker');
        this.isConnected = false;
        
        // Tentative de reconnexion après 5 secondes
        setTimeout(() => {
            if (!this.isConnected && this.client) {
                TeslaChargingLogger.writeToLog('🔄 MQTT: Tentative de reconnexion');
                this.client.reconnect();
            }
        }, 5000);
    }
    
    // Méthode améliorée pour une meilleure gestion des sources de données
    static getVehicleData(carId = '1') {
        // Obtenir les données MQTT brutes
        const mqttData = this.vehicleData[carId];
        
        // Si les données MQTT sont complètes, les utiliser
        if (mqttData && 
            mqttData.charge_state && 
            mqttData.charge_state.battery_level > 0 &&
            this.isDataFresh(300000)) { // 5 minutes max
            
            // Ajouter un marqueur de source et timestamp
            mqttData.data_source = 'mqtt';
            mqttData._timestamp = Date.now();
            return mqttData;
        }
        
        // Dernière version connue des données valides
        if (!this.lastValidData) {
            this.lastValidData = {};
        }
        
        // Si nous avons déjà des données MQTT valides en cache, les utiliser
        if (this.lastValidData[carId] && 
            this.lastValidData[carId].charge_state && 
            this.lastValidData[carId].charge_state.battery_level > 0) {
            
            console.log("INFO: Utilisation des données MQTT en cache");
            return this.lastValidData[carId];
        }
        
        // Tenter de récupérer depuis l'API Tesla en dernier recours
        try {
            const TeslaService = require('./tesla.service').TeslaService;
            const apiData = TeslaService.getLastReceiveData();
            
            // Si les données API sont valides, les utiliser
            if (apiData && 
                apiData.charge_state && 
                apiData.charge_state.battery_level > 0) {
                
                // Stocker comme dernières données valides
                apiData.data_source = 'api';
                this.lastValidData[carId] = apiData;
                return apiData;
            }
        } catch (e) {
            console.log("Impossible d'accéder aux données API Tesla");
        }
        
        // Si on arrive jusqu'ici, retourner les données MQTT telles quelles
        // (probablement incomplètes)
        return mqttData || { data_source: 'empty' };
    }

    // Vérifier fraîcheur des données
    static isDataFresh(maxAgeMs = 300000) { // 5 minutes par défaut
        return this.lastUpdateTime > 0 && 
               (Date.now() - this.lastUpdateTime) < maxAgeMs;
    }
    
    // Vérifier si attribut spécifique est frais
    static isAttributeFresh(attribute, maxAgeMs = 300000) {
        return this.lastStateUpdate[attribute] && 
               (Date.now() - this.lastStateUpdate[attribute]) < maxAgeMs;
    }
    
    // Ajout d'un listener pour les mises à jour
    static addUpdateListener(callback) {
        if (typeof callback === 'function') {
            this.updateCallbacks.push(callback);
        }
    }
    
    // Notification des listeners
    static notifyUpdateListeners(carId, attribute, value) {
        this.updateCallbacks.forEach(callback => {
            try {
                callback(carId, attribute, value);
            } catch (error) {
                console.error('Error in MQTT update listener:', error);
            }
        });
    }
}

module.exports = TeslaMqttClient;